package presenter.evaluation;

import model.Line;
import presenter.algorithms.util.FastElementSelector;

import java.util.ArrayList;
import java.util.LinkedList;

/**
 * Implementierung verschiedener Algorithmen zur Berechnung von Ausgleichsgeraden.
 *
 * @Author: Armin Wolf
 * @Email: a_wolf28@uni-muenster.de
 * @Date: 07.09.2017.
 */
public class ScaleDependentMeasure {

    private ArrayList<Double> errorValues;

    public ScaleDependentMeasure(final LinkedList<Line> lines, Double m, Double b) {
        //Liste mit den Fehler zu jedem Punkt
        errorValues = new ArrayList<>();

        for (Line line : lines) {
            Double e = Math.pow(m * line.getM() - line.getB() + b, 2) / (Math.pow(m, 2) + 1);
            errorValues.add(e);
        }
    }

    /* Skalierungs Abhängige Approximationsgüten */
    //unterschiedliche Alg.- auf einem Datensatz
    public Double mse() {
        double error = 0;

        for (Double d : errorValues) {
            error += Math.pow(d, 2);
        }

        error /= errorValues.size();

        return error;
    }

    public Double rmse() {
        return Math.sqrt(mse());
    }

    public Double mae() {
        double error = 0;
        for (Double d : errorValues) {
            error += Math.abs(d);
        }
        error /= errorValues.size();
        return error;
    }

    public Double mdae() {
        return FastElementSelector
                .randomizedSelect(errorValues, errorValues.size() * 0.5);
    }
}
